// file: $USFS/class/image_analysis/v6.1/imaga_ent_08.cc
//

// isip include files
//
#include "image_analysis.h"
#include "image_analysis_constants.h"

// method: ent_write_cc
//
// arguments: none
//
// return: a logical_1 value indicating status
//
// this method writes the entropy values of each of the color
//
logical_1 Image_analysis::ent_write_cc() {
  
  // define local variables
  //
  int_4 counter = 0;
  
  // open the file for writing the histogram model file
  //
  FILE* fp = open_cc(model_fname_d, ISIP_WRITE_ONLY);
  if (fp == (FILE*)NULL) {
    error_handler_cc((char_1*)"sd_write_cc",
		     (char_1*)"error opening the model file");
    return ISIP_FALSE;
  }
  
  for (int_4 i=0; i < IMAGA_ENT_MODEL_LINES; i++) {

    if (counter == 0) {
      
      // write the version into the file
      //
      fprintf(fp, "%s\n\n", (char*)IMAGA_MODEL_VERSION);
    }
    
    else if (counter == 1) {
      
      // write the type of the algorithm
      //
      fprintf(fp, (char*)IMAGA_HIST_COMMENT_FORMAT, IMAGA_HIST_COMMENT,
	      (char*)IMAGA_HIST_ALG_COMMENT, ISIP_POUND);
      fprintf(fp, "algorithm = %s\n\n", (char*)IMAGA_ALG_ENT_NAME);
    }
    
    else if (counter == 2) {

      // write the parameters of the histogram
      //
      fprintf(fp, (char*)IMAGA_HIST_COMMENT_FORMAT, IMAGA_HIST_COMMENT,
	      (char*)IMAGA_HIST_PARAM_COMMENT, ISIP_POUND);
      fprintf(fp, "min_value = %ld\nmax_val = %ld\nnum_bins = %ld\n\n",
	      (int_4)IMAGA_ENT_MIN, (int_4)IMAGA_ENT_MAX, IMAGA_ENT_NBINS);
    }

    else if (counter == 3) {
      
      // write the number of pixels and the images processed
      //
      fprintf(fp, (char*)IMAGA_HIST_COMMENT_FORMAT, IMAGA_HIST_COMMENT,
	      (char*)IMAGA_HIST_DATA_COMMENT, ISIP_POUND);
      fprintf(fp, "num_pixels = %ld\nnum_blocks_processed = %ld\n\n",
	      huf_r_npxls_d, huf_r_nblks_d);
    }

    else if (counter == 4) {
      
      // write the color weights
      //
      fprintf(fp, (char*)IMAGA_HIST_COMMENT_FORMAT, IMAGA_HIST_COMMENT,
	      (char*)IMAGA_HIST_CWTS_COMMENT, ISIP_POUND);
      fprintf(fp, "weight_red = %0.2f\nweight_green = %0.2f\nweight_blue = %0.2f\n\n",
	      IMAGA_ENT_WRED, IMAGA_ENT_WGRN, IMAGA_ENT_WBLU);
    }
    
    else if (counter == 5) {
      
      // write the scaling values
      //
      fprintf(fp, (char*)IMAGA_HIST_COMMENT_FORMAT, IMAGA_HIST_COMMENT,
	      (char*)IMAGA_HIST_SCAL_COMMENT, ISIP_POUND);
      fprintf(fp, "scale_a = %ld\nscale_b = %ld\n\n", IMAGA_ENT_ASCL,
	      IMAGA_ENT_BSCL);
    }
    
    else if (counter == 6) {
      
      // write the standard deviations 
      //
      fprintf(fp, (char*)IMAGA_HIST_COMMENT_FORMAT, IMAGA_HIST_COMMENT,
	      (char*)IMAGA_SD_VALUES_COMMENT, ISIP_POUND);
      fprintf(fp, "entropy_red = %f\nentropy_green = %f\n", huf_t_red_d,
	      huf_t_grn_d);
      fprintf(fp, "entropy_blue = %f\n", huf_t_blu_d);
    }
    counter++;
  }
  
  // close the file
  //
  if (close_cc(fp) == ISIP_FALSE) {
    error_handler_cc((char_1*)"ent_write_cc",
		     (char_1*)"error closing the file");
    return ISIP_FALSE;
  } 
  
  // exit gracefully
  //
  return ISIP_TRUE;
}

