// file: $USFS/class/image_analysis/v3.0/imaga_ent_09.cc
//

// isip include files
//
#include "image_analysis.h"
#include <integral_constants.h>

// method: hist_compare_cc
//
// arguments:
//  float_8& sbe_a: (output) sbe value
//
// return: a logical_1 value indicating status
//
// this method computes an sbe from two histograms.
//
logical_1 Image_analysis::ent_compare_cc(float_8& sbe_a) {
  
  // declare local variables
  //
  float_8 diff_r = (float_4)0.0;
  float_8 diff_g = (float_4)0.0;
  float_8 diff_b = (float_4)0.0;
  float_8 rms_scaled;
  
  // compare the data from the model file with the computed data
  //
  diff_r = huf_r_red_d - huf_t_red_d;
  diff_g = huf_r_grn_d - huf_t_grn_d;
  diff_b = huf_r_blu_d - huf_t_blu_d;
  
  // weight the contributions from each color
  //
  rms_scaled = huf_rwgt_d * diff_r + huf_gwgt_d * diff_g +
    huf_bwgt_d * diff_b;
  
  // compute the scaled sbe
  //
  sbe_a = huf_ascale_d * rms_scaled + huf_bscale_d;
  
  if (debug_level_d > ISIP_DEBUG_DETAILED) {
    fprintf(stdout,"<%s> rms error = %f: (r) %f (g) %f (b) %f\n",
	    "Image_analysis::ent_compare_cc",
	    rms_scaled, diff_r, diff_g, diff_b);
  }

  if (debug_level_d > ISIP_DEBUG_BRIEF) {
    fprintf(stdout,"<%s> sbe = %f\n",
	    "Image_analysis::ent_compare_cc", sbe_a);
  }
  
  // exit gracefully
  //
  return ISIP_TRUE;
}
