// file: $USFS/class/image_analysis/v3.1/imaga_he_02.cc
//

// isip include files
//
#include "image_analysis.h"
#include "image_analysis_constants.h"

// method: pca_test_cc
//
// arguments:
//  float_8& sbe_a: (output) the image's sbe value
//  char_1* image_fname_a: (input) the image's filename
//
// return: a logical_1 value indicating the status
//
// this method computes a histogram and compares it to the reference model.
//
logical_1 Image_analysis::pca_test_cc(float_8& sbe_a,
				       char_1* image_fname_a) {

  // declare local variables
  //
  int_4 k=0;

  // initialize the data related to the feature analysis
  //
  if (pca_init_cc() == ISIP_FALSE) {
    error_handler_cc((char_1*)"pca_test_cc",
		     (char_1*)"error while initializing the data");
    return ISIP_FALSE;
  }
  
  if ((sel_vec_d[0] == 1) || (sel_vec_d[10] == 1) || (sel_vec_d[20] == 1)
      || (sel_vec_d[45] == 1) || (sel_vec_d[55] == 1)) {

    // initialize the histogram data
    //
    if (hist_init_cc() == ISIP_FALSE) {
      error_handler_cc((char_1*)"pca_test_cc",
		       (char_1*)"error while initializing the histogram");
      return ISIP_FALSE;
    }
        
    // compute the histogram
    //
    if (hist_compute_cc(image_fname_a) == ISIP_FALSE) {
      error_handler_cc((char_1*)"pca_test_cc",
		       (char_1*)"error while computing the histogram");
      return ISIP_FALSE;
    }
    
    // normalize the data
    //
    if (hist_normalize_cc() == ISIP_FALSE) {
      error_handler_cc((char_1*)"pca_test_cc",
		       (char_1*)"error while normalizing the histogram");
      return ISIP_FALSE;
    }
  }

  if ((sel_vec_d[30] == 1) || (sel_vec_d[31] == 1)) {
      
    // initialize the edge detection data
    //
    if (edge_init_cc() == ISIP_FALSE) {
      error_handler_cc((char_1*)"pca_test_cc",
		       (char_1*)"error while initializing the edge detection");
      return ISIP_FALSE;
    }
  
    // perform the edge detection
    //
    if (edge_compute_cc(image_fname_a) == ISIP_FALSE) {
      error_handler_cc((char_1*)"pca_test_cc",
		       (char_1*)"error while computing the edges");
      return ISIP_FALSE;
    }
  
    // perform the normalization
    //
    if (edge_normalize_cc() == ISIP_FALSE) {
      error_handler_cc((char_1*)"pca_test_cc",
		       (char_1*)"error while normalizing the data");
      return ISIP_FALSE;
    }
  }

  if (sel_vec_d[32] == 1) {
  
    // initialize the parameters related to the sharpness
    //
    if (dent_init_cc() == ISIP_FALSE) {
      error_handler_cc((char_1*)"pca_test_cc",
		       (char_1*)"error initializing the sharpness paramaters");
      return ISIP_FALSE;
    }
    
    // compute the sharpness of the image
    //
    if (dent_compute_cc(image_fname_a) == ISIP_FALSE) {
      error_handler_cc((char_1*)"pca_test_cc",
		       (char_1*)"error while computing the sharpness");
      return ISIP_FALSE;
    }
  
    // normalizing the data
    //
    if (dent_normalize_cc() == ISIP_FALSE) {
      error_handler_cc((char_1*)"pca_test_cc",
		       (char_1*)"error while normalizing the sharpness");
      return ISIP_FALSE;
    }
  }

  if (sel_vec_d[35] == 1) {
  
    // initialize the standard deviation parameters
    //
    if (sd_init_cc() == ISIP_FALSE) {
      error_handler_cc((char_1*)"pca_test_cc",
		       (char_1*)"error intializing the SD parameters");
      return ISIP_FALSE;
    }
    
    // compute the standard deviation of the image
    //
    if (sd_compute_cc(image_fname_a) == ISIP_FALSE) {
      error_handler_cc((char_1*)"pca_test_cc",
		       (char_1*)"error computing the standard deviation");
      return ISIP_FALSE;
    }
  
    // normalize the values
    //
    if (sd_normalize_cc() == ISIP_FALSE) {
      error_handler_cc((char_1*)"pca_test_cc",
		       (char_1*)"error while normalizing the values");
      return ISIP_FALSE;
    }
  }

  if (sel_vec_d[38] == 1) {
  
    // initialize the entropy
    //
    if (ent_init_cc() == ISIP_FALSE) {
      error_handler_cc((char_1*)"pca_test_cc",
		       (char_1*)"error computing the entropy");
      return ISIP_FALSE;
    }

    // compute the entropy of the image
    //
    if (ent_compute_cc(image_fname_a) == ISIP_FALSE) {
      error_handler_cc((char_1*)"pca_test_cc",
		       (char_1*)"error while the standard deviation");
      return ISIP_FALSE;
    }
  }

  if (sel_vec_d[41] == 1) {
  
    // initializing the run length coding
    //
    if (rle_init_cc() == ISIP_FALSE) {
      error_handler_cc((char_1*)"pca_test_cc",
		       (char_1*)"error computing the run lenght coding");
      return ISIP_FALSE;
    }
    
    // computing the run length coding
    //
    if (rle_compute_cc(image_fname_a) == ISIP_FALSE) {
      error_handler_cc((char_1*)"pca_test_cc",
		       (char_1*)"error while computing the run lenght coding");
      return ISIP_FALSE;
    }
  }

  if (sel_vec_d[42] == 1) {
  
    // initialize the fractal dimension
    //
    if (fract_init_cc() == ISIP_FALSE) {
      error_handler_cc((char_1*)"pca_test_cc",
		       (char_1*)"error initialising the fractal dimension");
      return ISIP_FALSE;
    }
  
    // computing the fractal dimension
    //
    if (fract_compute_cc(image_fname_a) == ISIP_FALSE) {
      error_handler_cc((char_1*)"pca_test_cc",
		       (char_1*)"error computing the fractal dimesion");
      return ISIP_FALSE;
    }
  }
  
  // reading the values into the feature vector
  //
  for (int_4 i=0; i < pca_t_num_vectors_d; i++) {
    
    if (sel_vec_d[i] == 1) {
      
      // reading the red values
      //
      if (i < 10) {
	pca_t_data_d[k] = hist_t_rpdf_d[i];
	k++;
      }
      
      // reading the green values
      //
      if ((i >= 10) && (i < 20)) {
	pca_t_data_d[k] = hist_t_gpdf_d[i - 10];
	k++;
      }
      
      // reading the blue values
      //
      if ((i >= 20) && (i < 30)) {
	pca_t_data_d[k] = hist_t_bpdf_d[i - 20];
	k++;
      }

      // reading the long and short lines
      //
      if (i == 30) {
	pca_t_data_d[k] = edge_perlong_t_lines_d;
	k++;
      }
      
      if (i == 31) {
	pca_t_data_d[k] = edge_pershrt_t_lines_d;
	k++;
      }
      
      // reading the sharpness values
      //
      if (i == 32) {
	pca_t_data_d[k] = dent_t_sumred_d;
	k++;
      }
      
      if (i == 33) {
	pca_t_data_d[k] = dent_t_sumgrn_d;
	k++;
      }
      
      if (i == 34) {
	pca_t_data_d[k] = dent_t_sumblu_d;
	k++;
      }
      
      // reading the standard deviation
      //
      if (i == 35) {
	pca_t_data_d[k] = sd_t_red_d;
	k++;
      }
      
      if (i == 36) {
	pca_t_data_d[k] = sd_t_grn_d;
	k++;
      }
      
      if (i == 37) {
	pca_t_data_d[k] = sd_t_blu_d;
	k++;
      }
      
      // reading the entropy values
      //
      if (i == 38) {
	pca_t_data_d[k] = huf_t_red_d;
	k++;
      }

      if (i == 39) {
	pca_t_data_d[k] = huf_t_grn_d;
	k++;
      }

      if (i == 40) {
	pca_t_data_d[k] = huf_t_blu_d;
	k++;
      }

      // reading the compression ratio
      //
      if (i == 41) {
	pca_t_data_d[k] = rle_t_compression_d;
	k++;
      }

      // reading the fractal dimension
      //
      if (i == 42) {
	pca_t_data_d[k] = fract_t_dimred_d;
	k++;
      }
      
      if (i == 43) {
	pca_t_data_d[k] = fract_t_dimgrn_d;
	k++;
      }
      
      if (i == 44) {
	pca_t_data_d[k] = fract_t_dimblu_d;
	k++;
      }
      
      // reading the yellow values
      //
      if ((i >= 45) && (i < 55)) {
	pca_t_data_d[k] = hist_t_ylwpdf_d[i - 45];
	k++;
      }
      
      // reading the brown values
      //
      if ((i >= 55) && (i < 65)) {
	pca_t_data_d[k] = hist_t_brnpdf_d[i - 55];
	k++;
      }      
    }
  }

  // compute the sbe
  //
  if (pca_compare_cc(sbe_a) == ISIP_FALSE) {
    error_handler_cc((char_1*)"pca_test_cc",
		     (char_1*)"error while computing the sbe");
    return ISIP_FALSE;
  }
  
  // debugging information
  //
  if (debug_level_d > ISIP_DEBUG_DETAILED) {
    
    // printing the  feature vector information
    //
    fprintf(stdout, "<Image_analysis::pca_train_cc>\n");
    fprintf(stdout, "\n feature vector of the image\n");
    
    
    for (int_4 i=0; i < pca_t_values_read_d; i++) {
      fprintf(stdout, "%ld\t%f\n", i, pca_t_data_d[i]);
    }
  }
  
  // exit gracefully
  //
  return ISIP_TRUE;
}
