// file: $USFS/util/analyze_image/v1.0/ai_gp_0.cc
//
// a simple command line parser
//

// isip include files
//
#include "analyze_image.h"
#include "analyze_image_constants.h"

// function: get_parameters_cc
//
// arguments:
//  char_1*& param_file: (output) flag indicating if parameter option
//                                was entered
//  char_1*& output_file: (output) flag indicating if output option was
//                                 entered
//  int_4& debug_level: (output) flag indicating the debug level
//  int_4& pos: (output) position of where the first file appeared in
//                       the command line list
//  char_1** filenames: (output) list of files
//  int_4 argc: (input) argc from the main program
//  char_1** argv: (input) argv from the main program
//
// return value: a logical_1 indicating status
//
// This function parses command line arguments.
//
logical_1 get_parameters_cc(char_1*& param_file_a, char_1*& output_file_a,
			    int_4 &debug_level_a, int_4 &pos_a, int_4 argc,
			    char_1** argv) {
    
  // main decoding loop
  //
  int_4 last_index = argc;

  char_1 tmp_string[ISIP_MAX_STRING_LENGTH];
  
  // if no arguments are given then print out a help message
  //
  if (last_index == (int)1) {
    sprintf((char*)tmp_string,
	    (char*)ISIP_HELP_FORMAT, (char*)AI_HELP_FILE);
    system((char*)tmp_string);
  
    // exit gracefully
    //
    exit (ISIP_NO_ERROR);
  }

  for (int_4 index=1; index<last_index; index++) {

    // check for help
    //
    if (strcmp((char*)ISIP_HELP_OPTION, (char*)argv[index]) == (int)0) {
      sprintf((char*)tmp_string,
	      (char*)ISIP_HELP_FORMAT, (char*)AI_HELP_FILE);
      system((char*)tmp_string);

      // exit gracefully
      //
      exit(ISIP_NO_ERROR);
    }

    // pick off param option
    //
    else if (strcmp((char*)AI_OPT_A, (char*)argv[index]) == (int)0) {
      strcpy ((char*)tmp_string, (char*)argv[index+1]);
      param_file_a = new char_1[strlen((char*)tmp_string)+1];
      strcpy ((char*)param_file_a, (char*)tmp_string);
      pos_a = index+2;
    }

    // pick off output option
    //
    else if (strcmp((char*)AI_OPT_B, (char*)argv[index]) == (int)0) {
      strcpy ((char*)tmp_string, (char*)argv[index+1]);
      output_file_a = new char_1[strlen((char*)tmp_string)+1];
      strcpy ((char*)output_file_a, (char*)tmp_string);
      pos_a = index+2;
    }

    // pick off debug option
    //
    else if (strcmp((char*)AI_OPT_C, (char*)argv[index]) == (int)0) {

      // save the debug level
      //
      debug_level_a = (int_4)atoi((char*)argv[index+1]);
      pos_a = index+2;
    }
  }

  // exit gracefully
  //
  return ISIP_TRUE;
}
