// file: $USFS/util/segment_image/v1.0/si_gp_0.cc
//
// a simple command line parser
//

// isip include files
//
#include "segment_image.h"
#include "segment_image_constants.h"

// function: get_parameters_cc
//
// arguments:
//  char_1*& param_filelist: (output) flag indicating if parameter option
//                                was entered
//  char_1*& output_ext: (output) flag indicating if output option was
//                                 entered
//  char_1*& output_datafile: (output) the output data file name
//  int_4& debug_level: (output) flag indicating the debug level
//  char_1*& seg_filelist: (output) flag indicating if segmentation option
//                                  was entered
//  int_4& pos: (output) position of where the first file appeared in
//                         the command line list
//  char_1** filenames: (output) list of files
//  int_4 argc: (input) argc from the main program
//  char_1** argv: (input) argv from the main program
//
// return:
//  a logical_1 value indicating status
//
// This function parses command line arguments.
//
logical_1 get_parameters_cc(char_1*& param_filelist_a, char_1*& output_ext_a,
			    char_1*& output_datafile_a, int_4 &debug_level_a,
			    char_1*& seg_filelist_a,
			    int_4 &pos_a, int_4 argc, char_1** argv) {
    
  // main decoding loop
  //
  int_4 last_index = argc;

  char_1* tmp_string = new char_1[ISIP_MAX_STRING_LENGTH];
  
  // if no arguments are given then print out a help message
  //
  if (last_index == (int)1) {
    sprintf((char*)tmp_string,
	    (char*)ISIP_HELP_FORMAT, (char*)SI_HELP_FILE);
    system((char*)tmp_string);
  
    // exit gracefully
    //
    exit (ISIP_NO_ERROR);
  }

  for (int_4 index=1; index<last_index; index++) {

    // check for help
    //
    if (strcmp((char*)ISIP_HELP_OPTION, (char*)argv[index]) == (int)0) {
      sprintf((char*)tmp_string,
	      (char*)ISIP_HELP_FORMAT, (char*)SI_HELP_FILE);
      system((char*)tmp_string);

      // exit gracefully
      //
      exit(ISIP_NO_ERROR);
    }

    // pick off param option
    //
    else if (strcmp((char*)SI_OPT_A, (char*)argv[index]) == (int)0) {
      strcpy ((char*)tmp_string, (char*)argv[index+1]);
      param_filelist_a = new char_1[strlen((char*)tmp_string)+1];
      strcpy ((char*)param_filelist_a, (char*)tmp_string);
      pos_a = index+2;
    }

    // pick off out_ext option
    //
    else if (strcmp((char*)SI_OPT_B, (char*)argv[index]) == (int)0) {
      strcpy ((char*)tmp_string, (char*)argv[index+1]);
      output_ext_a = new char_1[strlen((char*)tmp_string)+1];
      strcpy ((char*)output_ext_a, (char*)tmp_string);
      pos_a = index+2;
    }

    // pick off debug option
    //
    else if (strcmp((char*)SI_OPT_C, (char*)argv[index]) == (int)0) {

      // save the debug level
      //
      debug_level_a = (int_4)atoi((char*)argv[index+1]);
      pos_a = index+2;
    }

    // pick off segmentation option
    //
    else if (strcmp((char*)SI_OPT_D, (char*)argv[index]) == (int)0) {
      strcpy ((char*)tmp_string, (char*)argv[index+1]);
      seg_filelist_a = new char_1[strlen((char*)tmp_string)+1];
      strcpy ((char*)seg_filelist_a, (char*)tmp_string);
      pos_a = index+2;
    }

    // pick off output_data option
    //
    else if (strcmp((char*)SI_OPT_E, (char*)argv[index]) == (int)0) {
      strcpy ((char*)tmp_string, (char*)argv[index+1]);
      output_datafile_a = new char_1[strlen((char*)tmp_string)+1];
      strcpy ((char*)output_datafile_a, (char*)tmp_string);
      pos_a = index+2;
    }
  }

  // delete memory
  //
  delete [] tmp_string;
  
  // exit gracefully
  //
  return ISIP_TRUE;
}
